"""
Final comprehensive test to reach 95% coverage
"""

import pytest
import os
import sys
import json
import tempfile
import shutil
from pathlib import Path
from unittest.mock import patch, MagicMock, mock_open, call, PropertyMock
from datetime import datetime, timedelta
from uuid import uuid4
import subprocess
import io
import time
import builtins


@pytest.fixture
def mock_syftbox_env(tmp_path):
    """Create a mock SyftBox environment"""
    syftbox_dir = tmp_path / "SyftBox" / "datasites" / "test@example.com" / "app_data" / "syft-queues"
    syftbox_dir.mkdir(parents=True)
    
    with patch('syft_queue.queue.get_queues_path', return_value=syftbox_dir):
        with patch('syft_queue.queue._detect_syftbox_queues_path', return_value=syftbox_dir):
            yield syftbox_dir


def test_server_utils_comprehensive():
    """Test all server_utils.py lines"""
    from syft_queue import server_utils
    
    # Test get_config_path
    config_path = server_utils.get_config_path()
    assert config_path == Path.home() / ".syftbox" / "syft_queue.config"
    
    # Test read_config - config exists and valid
    with patch('pathlib.Path.exists', return_value=True):
        with patch('builtins.open', mock_open(read_data='{"port": 8080}')):
            config = server_utils.read_config()
            assert config == {"port": 8080}
    
    # Config exists but invalid JSON
    with patch('pathlib.Path.exists', return_value=True):
        with patch('builtins.open', side_effect=Exception("Invalid JSON")):
            config = server_utils.read_config()
            assert config == {}
    
    # Config doesn't exist
    with patch('pathlib.Path.exists', return_value=False):
        config = server_utils.read_config()
        assert config == {}
    
    # Test get_syft_queue_url - with config
    with patch('syft_queue.server_utils.read_config', return_value={"port": 9000}):
        url = server_utils.get_syft_queue_url()
        assert "9000" in url
        
        # With endpoint
        url = server_utils.get_syft_queue_url("health")
        assert "health" in url
    
    # With port file (backward compatibility)
    with patch('syft_queue.server_utils.read_config', return_value={}):
        with patch('pathlib.Path.exists', return_value=True):
            with patch('pathlib.Path.read_text', return_value='8888'):
                url = server_utils.get_syft_queue_url()
                assert "8888" in url
    
    # With port file error
    with patch('syft_queue.server_utils.read_config', return_value={}):
        with patch('pathlib.Path.exists', return_value=True):
            with patch('pathlib.Path.read_text', side_effect=Exception("Read error")):
                url = server_utils.get_syft_queue_url()
    
    # With environment variables
    with patch('syft_queue.server_utils.read_config', return_value={}):
        with patch('pathlib.Path.exists', return_value=False):
            with patch.dict('os.environ', {'SYFTQUEUE_PORT': '7777'}, clear=True):
                url = server_utils.get_syft_queue_url()
                assert "7777" in url
    
    # Test is_server_running
    with patch('requests.get') as mock_get:
        mock_get.return_value = MagicMock(status_code=200)
        assert server_utils.is_server_running() is True
        
        mock_get.return_value = MagicMock(status_code=500)
        assert server_utils.is_server_running() is False
        
        mock_get.side_effect = Exception("Connection error")
        assert server_utils.is_server_running() is False
    
    # Test is_syftbox_mode
    with patch('syft_queue.server_utils.is_syftbox_installed', return_value=True):
        with patch('syft_queue.server_utils.is_app_installed', return_value=True):
            assert server_utils.is_syftbox_mode() is True
    
    # Import error
    with patch('syft_queue.server_utils.is_syftbox_installed', side_effect=ImportError):
        assert server_utils.is_syftbox_mode() is False
    
    # Test ensure_server_healthy
    with patch('syft_queue.server_utils.is_server_running', side_effect=[False, False, True]):
        with patch('time.sleep'):
            assert server_utils.ensure_server_healthy() is True
    
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('time.sleep'):
            assert server_utils.ensure_server_healthy() is False
    
    # Test start_server - already running
    with patch('syft_queue.server_utils.is_server_running', return_value=True):
        assert server_utils.start_server() is True
    
    # SyftBox mode - healthy
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=True):
            with patch('syft_queue.server_utils.ensure_server_healthy', return_value=True):
                assert server_utils.start_server() is True
    
    # SyftBox mode - not healthy
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=True):
            with patch('syft_queue.server_utils.ensure_server_healthy', return_value=False):
                with patch('warnings.warn'):
                    assert server_utils.start_server() is False
    
    # Non-SyftBox mode - script not found
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=False):
            with patch('pathlib.Path.exists', return_value=False):
                with patch('builtins.print'):
                    assert server_utils.start_server() is False
    
    # Non-SyftBox mode - successful start
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=False):
            with patch('pathlib.Path.exists', return_value=True):
                with patch('subprocess.Popen'):
                    with patch('syft_queue.server_utils.ensure_server_healthy', return_value=True):
                        assert server_utils.start_server() is True
    
    # Start fails with timeout
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=False):
            with patch('pathlib.Path.exists', return_value=True):
                with patch('subprocess.Popen'):
                    with patch('syft_queue.server_utils.ensure_server_healthy', return_value=False):
                        with patch('builtins.print'):
                            assert server_utils.start_server() is False
    
    # Exception during start
    with patch('syft_queue.server_utils.is_server_running', return_value=False):
        with patch('syft_queue.server_utils.is_syftbox_mode', return_value=False):
            with patch('pathlib.Path.exists', return_value=True):
                with patch('subprocess.Popen', side_effect=Exception("Failed to start")):
                    with patch('builtins.print'):
                        assert server_utils.start_server() is False


def test_init_coverage():
    """Test __init__.py coverage"""
    # Clear modules to test import
    for mod in list(sys.modules.keys()):
        if mod.startswith('syft_queue'):
            del sys.modules[mod]
    
    # Test auto_install exception
    with patch('syft_queue.auto_install.auto_install', side_effect=Exception("Install error")):
        import syft_queue
    
    # Test banner in interactive mode
    with patch('sys.flags') as mock_flags:
        mock_flags.interactive = True
        with patch('syft_queue.auto_install.show_startup_banner') as mock_banner:
            # Re-import to trigger banner
            pass
    
    # Test queues collection
    from syft_queue import queues
    
    # Test __repr__ in Jupyter
    queues._ipython_canary_method_should_not_exist_ = True
    with patch.object(queues, '_repr_html_', return_value='<html>Test</html>'):
        result = repr(queues)
        assert result == '<html>Test</html>'
    del queues._ipython_canary_method_should_not_exist_
    
    # Test __repr__ non-Jupyter
    from syft_queue.queue import _get_queues_table
    with patch('syft_queue.queue._get_queues_table', return_value='Table'):
        result = repr(queues)
        assert result == 'Table'
    
    # Test __str__
    with patch('syft_queue.queue._get_queues_table', return_value='String'):
        result = str(queues)
        assert result == 'String'
    
    # Test _repr_html_ - server fails
    with patch('syft_queue.server_utils.start_server', return_value=False):
        html = queues._repr_html_()
        assert 'Error' in html
    
    # Test _repr_html_ - server starts
    with patch('syft_queue.server_utils.start_server', return_value=True):
        with patch('syft_queue.server_utils.get_syft_queue_url', return_value='http://test/widget'):
            html = queues._repr_html_()
            assert 'iframe' in html
    
    # Test widget - server fails
    with patch('syft_queue.server_utils.start_server', return_value=False):
        widget = queues.widget()
        assert 'Error' in widget
    
    # Test widget - server starts
    with patch('syft_queue.server_utils.start_server', return_value=True):
        with patch('syft_queue.server_utils.get_syft_queue_url', return_value='http://test/widget'):
            widget = queues.widget(width="800px", height="600px", url="http://custom")
            assert 'width="800px"' in widget
            assert 'http://custom' in widget
    
    # Test cleanup on import - this happens during import
    # Test test_utils import
    try:
        from syft_queue import cleanup_all_test_artifacts
    except ImportError:
        pass


def test_queue_path_detection(tmp_path):
    """Test _detect_syftbox_queues_path"""
    from syft_queue.queue import _detect_syftbox_queues_path
    
    # Test SYFTBOX_DATA_FOLDER
    with patch.dict('os.environ', {'SYFTBOX_DATA_FOLDER': str(tmp_path / 'data')}, clear=True):
        with patch('pathlib.Path.mkdir'):
            result = _detect_syftbox_queues_path()
            assert str(result) == str(tmp_path / 'data')
    
    # Test SYFTBOX_EMAIL
    syftbox_dir = tmp_path / 'SyftBox' / 'datasites' / 'test@example.com' / 'app_data' / 'syft-queues'
    syftbox_dir.mkdir(parents=True)
    
    with patch.dict('os.environ', {'SYFTBOX_EMAIL': 'test@example.com'}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            result = _detect_syftbox_queues_path()
            assert 'test@example.com' in str(result)
    
    # Test YAML config
    config_dir = tmp_path / '.syftbox'
    config_dir.mkdir(exist_ok=True)
    config_file = config_dir / 'config.yaml'
    config_file.write_text('email: yaml@test.com')
    
    yaml_syftbox = tmp_path / 'SyftBox' / 'datasites' / 'yaml@test.com' / 'app_data' / 'syft-queues'
    yaml_syftbox.mkdir(parents=True)
    
    with patch.dict('os.environ', {}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            result = _detect_syftbox_queues_path()
            assert 'yaml@test.com' in str(result)
    
    # Test YAML ImportError with manual parsing
    with patch.dict('os.environ', {}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            original_import = builtins.__import__
            def mock_import(name, *args, **kwargs):
                if name == 'yaml':
                    raise ImportError()
                return original_import(name, *args, **kwargs)
            
            with patch('builtins.__import__', side_effect=mock_import):
                result = _detect_syftbox_queues_path()
                assert 'yaml@test.com' in str(result)
    
    # Test config error
    config_file.write_text('invalid yaml: {{{')
    with patch.dict('os.environ', {}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            result = _detect_syftbox_queues_path()
    
    # Test git config
    config_file.unlink()
    git_syftbox = tmp_path / 'SyftBox' / 'datasites' / 'git@test.com' / 'app_data' / 'syft-queues'
    git_syftbox.mkdir(parents=True)
    
    with patch.dict('os.environ', {}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            with patch('subprocess.run') as mock_run:
                mock_run.return_value = MagicMock(returncode=0, stdout='git@test.com')
                result = _detect_syftbox_queues_path()
                assert 'git@test.com' in str(result)
    
    # Test git fails
    with patch.dict('os.environ', {}, clear=True):
        with patch('pathlib.Path.home', return_value=tmp_path):
            with patch('subprocess.run', side_effect=subprocess.CalledProcessError(1, 'git')):
                result = _detect_syftbox_queues_path()
                assert result == Path.cwd()


def test_job_class_comprehensive(mock_syftbox_env):
    """Test Job class comprehensively"""
    from syft_queue import q, JobStatus, Job
    
    queue = q("job_test", force=True)
    
    # Test create_job with all parameters
    job = queue.create_job(
        name="test_job",
        requester_email="user@test.com",
        target_email="owner@test.com",
        description="Test description",
        data={"input": "data"},
        mock_data=True,
        metadata={"priority": "high"}
    )
    
    # Test _make_relative - relative path
    rel_path = job.object_path / "subdir" / "file.txt"
    rel_path.parent.mkdir(parents=True, exist_ok=True)
    rel_path.touch()
    
    # Mock is_relative_to if not available
    if not hasattr(Path, 'is_relative_to'):
        Path.is_relative_to = lambda self, other: str(other) in str(self)
    
    result = job._make_relative(rel_path)
    assert isinstance(result, (str, Path))
    
    # Test _make_relative - absolute path
    abs_path = Path("/absolute/path")
    result = job._make_relative(abs_path)
    assert result == abs_path
    
    # Test update_relative_paths
    job.code_folder = str(job.object_path / "code")
    (job.object_path / "code").mkdir()
    job.update_relative_paths()
    
    # Test __str__
    job.description = "Test description"
    str_result = str(job)
    assert job.name in str_result
    
    # Test resolved_code_folder - exists
    code_dir = mock_syftbox_env / "code"
    code_dir.mkdir()
    job.code_folder = str(code_dir)
    assert job.resolved_code_folder == code_dir
    
    # Test resolved_code_folder - doesn't exist
    job.code_folder = str(mock_syftbox_env / "nonexistent")
    result = job.resolved_code_folder
    
    # Test resolved_code_folder - relative
    job.code_folder = None
    job.code_folder_relative = "rel_code"
    rel_dir = job.object_path / "rel_code"
    rel_dir.mkdir(parents=True, exist_ok=True)
    assert job.resolved_code_folder == rel_dir
    
    # Test resolved_code_folder - absolute fallback
    job.code_folder_relative = None
    job.code_folder_absolute_fallback = str(code_dir)
    assert job.resolved_code_folder == code_dir
    
    # Test resolved_code_folder - search in job dir
    job.code_folder_absolute_fallback = None
    default_code = job.object_path / "code"
    default_code.mkdir(exist_ok=True)
    result = job.resolved_code_folder
    
    # Test resolved_output_folder
    output_dir = mock_syftbox_env / "output"
    output_dir.mkdir()
    job.output_folder = str(output_dir)
    assert job.resolved_output_folder == output_dir
    
    # Test resolved_output_folder - relative
    job.output_folder = None
    job.output_folder_relative = "rel_output"
    rel_output = job.object_path / "rel_output"
    rel_output.mkdir(parents=True, exist_ok=True)
    result = job.resolved_output_folder
    
    # Test is_expired
    job.updated_at = None
    assert not job.is_expired
    
    job.updated_at = datetime.now() - timedelta(days=31)
    assert job.is_expired
    
    job.updated_at = datetime.now() - timedelta(days=1)
    assert not job.is_expired
    
    # Test code_files
    (code_dir / "file1.py").write_text("code")
    (code_dir / "file2.py").write_text("code")
    job.code_folder = str(code_dir)
    files = job.code_files
    assert len(files) >= 2
    
    # Test code_files - no code folder
    job.code_folder = None
    files = job.code_files
    assert files == []
    
    # Test save/load
    job.save()
    
    # Test load with valid JSON
    loaded = Job(job.object_path, owner_email="owner@test.com")
    assert loaded.uid == job.uid
    
    # Test load with invalid JSON
    json_file = job.object_path / "private" / "job_data.json"
    json_file.write_text("{invalid")
    loaded = Job(job.object_path, owner_email="owner@test.com")
    
    # Test load with no JSON
    json_file.unlink()
    loaded = Job(job.object_path, owner_email="owner@test.com")
    
    # Test update_status
    new_job = queue.create_job("status", "user@test.com", "owner@test.com")
    new_job.update_status(JobStatus.approved)
    
    # Test update_status without queue ref
    new_job._queue_ref = None
    new_job.update_status(JobStatus.running)
    
    # Test terminal transitions
    new_job.update_status(JobStatus.completed)
    new_job.update_status(JobStatus.failed)
    
    # Test delete with error
    with patch('shutil.rmtree', side_effect=OSError("Permission denied")):
        try:
            new_job.delete()
        except OSError:
            pass
    
    # Test properties
    assert isinstance(job.is_terminal, bool)
    assert isinstance(job.is_approved, bool)
    assert isinstance(job.is_running, bool)
    assert isinstance(job.is_completed, bool)
    assert isinstance(job.is_failed, bool)
    assert isinstance(job.is_rejected, bool)
    
    # Test __repr__
    repr_str = repr(job)
    assert "Job" in repr_str


def test_queue_class_comprehensive(mock_syftbox_env):
    """Test Queue class comprehensively"""
    from syft_queue import q, JobStatus, DataQueue, get_queue
    
    # Test create queue
    queue = q("test_queue", force=True)
    
    # Test atomic creation with temp dir
    with patch('tempfile.mkdtemp', return_value=str(mock_syftbox_env / "temp")):
        with patch('pathlib.Path.rename'):
            queue2 = q("atomic_test", force=True)
    
    # Test create jobs
    jobs = []
    for i in range(5):
        job = queue.create_job(f"job{i}", f"user{i}@test.com", "owner@test.com")
        if i % 2 == 0:
            job.update_status(JobStatus.approved)
        jobs.append(job)
    
    # Test list_jobs
    all_jobs = queue.list_jobs()
    assert len(all_jobs) >= 5
    
    approved = queue.list_jobs(status=JobStatus.approved)
    assert len(approved) >= 2
    
    # Test get_job
    found = queue.get_job(jobs[0].uid)
    assert found is not None
    assert found.uid == jobs[0].uid
    
    not_found = queue.get_job(str(uuid4()))
    assert not_found is None
    
    # Test _move_job
    job_to_move = jobs[0]
    original_path = job_to_move.object_path
    queue._move_job(job_to_move, JobStatus.inbox, JobStatus.approved)
    
    # Test _move_job with error
    with patch('shutil.move', side_effect=OSError("Move failed")):
        try:
            queue._move_job(jobs[1], JobStatus.inbox, JobStatus.approved)
        except OSError:
            pass
    
    # Test create_job with all parameters
    complex_job = queue.create_job(
        "complex_job",
        "user@test.com",
        "owner@test.com",
        description="Complex test job",
        data={"real": "data", "nested": {"value": 123}},
        mock_data=True,
        metadata={"priority": "high", "tags": ["test", "complex"]}
    )
    
    # Test stats
    stats = queue.get_stats()
    assert "total_jobs" in stats
    assert "inbox" in stats
    assert "approved" in stats
    
    # Test refresh_stats
    queue.refresh_stats()
    
    # Test _update_stats
    queue._update_stats("inbox", -1)
    queue._update_stats("approved", 1)
    
    # Test __str__ and __repr__
    str_result = str(queue)
    assert "test_queue" in str_result
    
    repr_result = repr(queue)
    assert "Queue" in repr_result
    
    # Test DataQueue
    data_queue = q("data_queue", queue_type="data", force=True)
    assert isinstance(data_queue, DataQueue)
    
    # Test get_queue
    found_queue = get_queue("test_queue")
    assert found_queue is not None
    assert found_queue.name == "test_queue"
    
    not_found_queue = get_queue("nonexistent")
    assert not_found_queue is None


def test_progression_api_comprehensive(mock_syftbox_env):
    """Test progression API comprehensively"""
    from syft_queue import (
        q, approve, reject, start, complete, fail, timeout, advance,
        approve_all, process_queue, JobStatus
    )
    
    queue = q("prog_test", force=True)
    
    # Test approve
    job1 = queue.create_job("approve1", "user@test.com", "owner@test.com")
    approved = approve(job1, approver="admin@test.com", notes="Approved for testing")
    assert approved.status == JobStatus.approved
    
    # Test approve validation - already approved
    try:
        approve(approved)
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test approve - terminal state
    job1.update_status(JobStatus.completed)
    try:
        approve(job1)
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test reject
    job2 = queue.create_job("reject1", "user@test.com", "owner@test.com")
    rejected = reject(job2, reason="Invalid request", reviewer="admin@test.com")
    assert rejected.status == JobStatus.rejected
    
    # Test reject validation - already terminal
    try:
        reject(rejected, reason="Already rejected")
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test start
    job3 = queue.create_job("start1", "user@test.com", "owner@test.com")
    job3.update_status(JobStatus.approved)
    started = start(job3, runner="worker1", notes="Starting job")
    assert started.status == JobStatus.running
    
    # Test start validation - not approved
    job3_bad = queue.create_job("start_bad", "user@test.com", "owner@test.com")
    try:
        start(job3_bad)
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test complete
    completed = complete(started, output="Success output", duration_seconds=10)
    assert completed.status == JobStatus.completed
    
    # Test complete validation - not running
    job4 = queue.create_job("complete_bad", "user@test.com", "owner@test.com")
    try:
        complete(job4)
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test fail
    job5 = queue.create_job("fail1", "user@test.com", "owner@test.com")
    job5.update_status(JobStatus.running)
    failed = fail(job5, error="Process failed", exit_code=1)
    assert failed.status == JobStatus.failed
    
    # Test fail validation - not running
    job5_bad = queue.create_job("fail_bad", "user@test.com", "owner@test.com")
    try:
        fail(job5_bad, error="Cannot fail")
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test timeout
    job6 = queue.create_job("timeout1", "user@test.com", "owner@test.com")
    job6.update_status(JobStatus.running)
    job6.started_at = datetime.now() - timedelta(minutes=5)
    timed_out = timeout(job6)
    assert timed_out.status == JobStatus.failed
    
    # Test timeout validation - not running
    job6_bad = queue.create_job("timeout_bad", "user@test.com", "owner@test.com")
    try:
        timeout(job6_bad)
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test advance
    job7 = queue.create_job("advance1", "user@test.com", "owner@test.com")
    advanced = advance(job7)
    assert advanced.status == JobStatus.approved
    
    # Test advance - already terminal
    job7.update_status(JobStatus.completed)
    advanced_again = advance(job7)
    assert advanced_again.status == JobStatus.completed  # No change
    
    # Test approve_all
    batch = [queue.create_job(f"batch{i}", "user@test.com", "owner@test.com") for i in range(3)]
    approved_batch = approve_all(batch, approver="admin@test.com")
    assert len(approved_batch) == 3
    for job in approved_batch:
        assert job.status == JobStatus.approved
    
    # Test approve_all with mixed states
    mixed_batch = [queue.create_job(f"mixed{i}", "user@test.com", "owner@test.com") for i in range(3)]
    mixed_batch[0].update_status(JobStatus.approved)  # Already approved
    mixed_batch[1].update_status(JobStatus.completed)  # Terminal
    approved_mixed = approve_all(mixed_batch, approver="admin@test.com")
    # Should only approve the inbox job
    
    # Test process_queue
    for job in approved_batch:
        # Add code to jobs
        code_dir = job.object_path / "code"
        code_dir.mkdir()
        (code_dir / "run.py").write_text("print('test')")
        job.code_folder = str(code_dir)
        job.save()
    
    results = process_queue(queue, max_jobs=2)
    assert len(results) <= 2
    
    # Test process_queue with no jobs
    empty_queue = q("empty_queue", force=True)
    empty_results = process_queue(empty_queue)
    assert len(empty_results) == 0


def test_pipeline_comprehensive():
    """Test pipeline functionality comprehensively"""
    from syft_queue.pipeline import (
        Pipeline, PipelineBuilder, PipelineStage, PipelineTransition,
        advance_job, approve_job, reject_job, start_job, complete_job, fail_job,
        advance_jobs, validate_data_schema, check_model_performance,
        allocate_gpu_resources, register_model_endpoint,
        example_simple_approval_flow, example_complex_ml_pipeline,
        example_review_queue_batch_operations
    )
    
    # Test PipelineStage enum
    for stage in PipelineStage:
        assert isinstance(stage.value, str)
        assert stage.value in ["inbox", "approved", "running", "completed", "failed", "rejected"]
    
    # Test PipelineBuilder
    builder = PipelineBuilder("test_pipeline")
    
    # Test stage method
    builder.stage("inbox", "inbox", path=Path("/tmp/inbox"))
    builder.stage("review", "approved")
    builder.stage("process", "running")
    builder.stage("done", "completed")
    
    assert "inbox" in builder.stages
    assert "review" in builder.stages
    
    # Test transition method
    builder.transition("inbox", "review", condition=lambda j: True)
    builder.transition("review", "process")
    builder.transition("process", "done")
    
    assert len(builder.transitions) == 3
    
    # Test build method
    pipeline = builder.build()
    assert isinstance(pipeline, Pipeline)
    
    # Test Pipeline methods
    mock_job = MagicMock()
    mock_job.status = "inbox"
    mock_job.uid = "test-123"
    mock_job.object_path = Path("/tmp/job")
    mock_job.advance = MagicMock()
    
    # Test get_job_stage
    stage = pipeline.get_job_stage(mock_job)
    assert stage == "inbox"
    
    # Test add_stage
    pipeline.add_stage("extra", "extra", path=Path("/tmp/extra"))
    assert "extra" in pipeline.stages
    
    # Test add_transition
    pipeline.add_transition("done", "extra")
    
    # Test add_conditional_transition
    pipeline.add_conditional_transition("extra", "inbox", lambda j: j.status == "extra")
    
    # Test advance - basic
    with patch('pathlib.Path.exists', return_value=False):
        result = pipeline.advance(mock_job)
        mock_job.advance.assert_called_with("review")
    
    # Test advance - to specific stage
    mock_job.advance.reset_mock()
    result = pipeline.advance(mock_job, to_stage="process")
    mock_job.advance.assert_called_with("process")
    
    # Test advance - with path movement
    mock_job.advance.reset_mock()
    pipeline.stage_paths["review"] = Path("/tmp/review")
    with patch('pathlib.Path.exists', return_value=True):
        with patch('shutil.move') as mock_move:
            result = pipeline.advance(mock_job, to_stage="review")
            mock_move.assert_called_once()
    
    # Test advance - no valid transitions
    mock_job.status = "unknown"
    result = pipeline.advance(mock_job)
    
    # Test advance - condition returns False
    mock_job.status = "inbox"
    pipeline.transitions = [
        PipelineTransition("inbox", "review", condition=lambda j: False)
    ]
    result = pipeline.advance(mock_job)
    
    # Test _execute_transition
    transition = PipelineTransition("inbox", "review")
    pipeline._execute_transition(mock_job, transition)
    
    # Test _execute_transition with stage handler
    pipeline.stage_handlers["inbox"] = lambda j: print("handled")
    pipeline._execute_transition(mock_job, transition)
    
    # Test _execute_transition with hook
    pipeline.hooks[("inbox", "review")] = lambda j: print("hooked")
    pipeline._execute_transition(mock_job, transition)
    
    # Test job helper functions
    advance_job(mock_job)
    approve_job(mock_job)
    reject_job(mock_job, "test reason")
    start_job(mock_job)
    complete_job(mock_job)
    fail_job(mock_job, "test error")
    
    # Test advance_jobs
    jobs = [mock_job, mock_job]
    results = advance_jobs(jobs)
    assert len(results) == 2
    
    # Test advance_jobs with exception
    mock_job.advance.side_effect = Exception("Advance failed")
    results = advance_jobs([mock_job])
    mock_job.advance.side_effect = None
    
    # Test validator functions
    assert validate_data_schema(mock_job) is True
    assert check_model_performance(mock_job) is True
    assert allocate_gpu_resources(mock_job) is True
    register_model_endpoint(mock_job)
    
    # Test example functions
    example_simple_approval_flow()
    
    # Test complex example with mocking
    with patch('syft_queue.q') as mock_q:
        mock_queue = MagicMock()
        mock_q.return_value = mock_queue
        mock_queue.create_job.return_value = mock_job
        
        example_complex_ml_pipeline()
        mock_queue.create_job.assert_called()
    
    # Test batch operations example
    with patch('syft_queue.q') as mock_q:
        mock_queue = MagicMock()
        mock_q.return_value = mock_queue
        mock_queue.list_jobs.return_value = [mock_job] * 5
        
        example_review_queue_batch_operations()
        mock_queue.list_jobs.assert_called()


def test_utility_functions_comprehensive(mock_syftbox_env):
    """Test utility functions comprehensively"""
    from syft_queue import (
        list_queues, cleanup_orphaned_queues, recreate_missing_queue_directories,
        get_queues_path, help
    )
    from syft_queue.queue import (
        _queue_exists, _cleanup_empty_queue_directory, _is_ghost_job_folder,
        _cleanup_ghost_job_folders, _cleanup_all_ghost_job_folders,
        _queue_has_valid_syftobject, _cleanup_orphaned_queue_directories,
        _cleanup_all_orphaned_queue_directories, _get_queues_table,
        prepare_job_for_execution, execute_job_with_context
    )
    
    # Test _queue_exists
    with patch('syft_objects.objects.get_object', return_value=None):
        assert not _queue_exists("nonexistent")
    
    with patch('syft_objects.objects.get_object', return_value={"name": "Q:exists"}):
        assert _queue_exists("exists")
    
    # Test _cleanup_empty_queue_directory
    empty_dir = mock_syftbox_env / "Q:empty"
    empty_dir.mkdir()
    _cleanup_empty_queue_directory(empty_dir)
    
    # Test _is_ghost_job_folder
    ghost = mock_syftbox_env / "J:ghost"
    ghost.mkdir()
    assert _is_ghost_job_folder(ghost)
    
    not_ghost = mock_syftbox_env / "Q:queue"
    not_ghost.mkdir()
    assert not _is_ghost_job_folder(not_ghost)
    
    # Test _cleanup_ghost_job_folders
    queue_dir = mock_syftbox_env / "Q:cleanup"
    queue_dir.mkdir()
    (queue_dir / "J:ghost1").mkdir()
    (queue_dir / "J:ghost2").mkdir()
    (queue_dir / "inbox").mkdir()  # Not a ghost
    
    count = _cleanup_ghost_job_folders(queue_dir)
    assert count >= 2
    
    # Test _cleanup_all_ghost_job_folders
    total = _cleanup_all_ghost_job_folders()
    assert isinstance(total, int)
    
    # Test _queue_has_valid_syftobject
    with patch('syft_objects.objects.get_object', return_value={"name": "Q:valid"}):
        assert _queue_has_valid_syftobject("valid")
    
    with patch('syft_objects.objects.get_object', return_value=None):
        assert not _queue_has_valid_syftobject("invalid")
    
    # Test _cleanup_orphaned_queue_directories
    orphaned = mock_syftbox_env / "Q:orphaned"
    orphaned.mkdir()
    
    count = _cleanup_orphaned_queue_directories(mock_syftbox_env)
    assert isinstance(count, int)
    
    # Test _cleanup_all_orphaned_queue_directories
    total = _cleanup_all_orphaned_queue_directories()
    assert isinstance(total, int)
    
    # Test high-level functions
    queues = list_queues()
    assert isinstance(queues, list)
    
    cleanup_orphaned_queues()
    
    path = get_queues_path()
    assert isinstance(path, Path)
    
    recreate_missing_queue_directories()
    
    # Test _get_queues_table
    table = _get_queues_table()
    assert isinstance(table, str)
    
    # Test help
    with patch('builtins.print') as mock_print:
        help()
        mock_print.assert_called()
    
    # Test job execution functions
    from syft_queue import q
    queue = q("exec_test", force=True)
    
    # Create job with code
    code_dir = mock_syftbox_env / "code"
    code_dir.mkdir()
    (code_dir / "run.py").write_text("print('test')")
    
    job = queue.create_job("exec_job", "user@test.com", "owner@test.com")
    job.code_folder = str(code_dir)
    job.save()
    
    # Test prepare_job_for_execution
    context = prepare_job_for_execution(job)
    assert "job_uid" in context
    assert "job_name" in context
    
    # Test execute_job_with_context
    success, output = execute_job_with_context(job)
    assert isinstance(success, bool)
    assert isinstance(output, str)
    
    # Test execute_job_with_context with custom runner
    success, output = execute_job_with_context(job, runner_command="python")
    
    # Test execute_job_with_context with error
    with patch('subprocess.run', side_effect=Exception("Run failed")):
        success, output = execute_job_with_context(job)
        assert not success
        assert "error" in output.lower()


def test_comprehensive_integration(mock_syftbox_env):
    """Integration test covering remaining edge cases"""
    from syft_queue import (
        q, JobStatus, approve_all, get_queue, queue as queue_factory,
        create_job, create_queue
    )
    
    # Test queue factory validation
    try:
        queue_factory("test", queue_type="invalid")
        assert False, "Should have raised ValueError"
    except ValueError:
        pass
    
    # Test create_queue and create_job factory functions
    queue = create_queue("factory_test", force=True)
    job = create_job("factory_job", "user@test.com", "owner@test.com", queue="factory_test")
    
    # Create comprehensive test scenario
    queues = []
    for i in range(3):
        queue = q(f"integration_{i}", queue_type="code" if i < 2 else "data", force=True)
        queues.append(queue)
    
    # Create jobs with all configurations
    all_jobs = []
    for queue in queues:
        for j in range(3):
            job = queue.create_job(
                f"job_{j}",
                f"user{j}@test.com",
                "owner@test.com",
                description=f"Integration Job {j}",
                data={"index": j, "test": True} if j % 2 == 0 else None,
                mock_data=j % 3 == 0,
                metadata={"priority": j, "integration": True}
            )
            all_jobs.append(job)
    
    # Exercise all progression paths
    for i, job in enumerate(all_jobs):
        if i % 5 == 0:
            job.update_status(JobStatus.rejected)
        elif i % 5 == 1:
            job.update_status(JobStatus.approved)
            job.update_status(JobStatus.running)
            job.update_status(JobStatus.completed)
        elif i % 5 == 2:
            job.update_status(JobStatus.approved)
            job.update_status(JobStatus.running)
            job.update_status(JobStatus.failed)
        elif i % 5 == 3:
            job.update_status(JobStatus.approved)
        # Leave some in inbox
    
    # Test batch operations
    inbox_jobs = [j for j in all_jobs if j.status == JobStatus.inbox]
    if inbox_jobs:
        approve_all(inbox_jobs[:2])
    
    # Test queue operations
    for queue in queues:
        queue.refresh_stats()
        stats = queue.get_stats()
        jobs = queue.list_jobs()
        
        # Test with different status filters
        approved_jobs = queue.list_jobs(status=JobStatus.approved)
        completed_jobs = queue.list_jobs(status=JobStatus.completed)
    
    # Test finding queues
    for i in range(3):
        found = get_queue(f"integration_{i}")
        assert found is not None
    
    # Test edge cases
    edge_queue = q("edge_cases", force=True)
    
    # Job with minimal data
    minimal_job = edge_queue.create_job("minimal", "user@test.com", "owner@test.com")
    
    # Job with maximum data
    max_job = edge_queue.create_job(
        "maximum",
        "user@test.com",
        "owner@test.com",
        description="A" * 1000,  # Long description
        data={"complex": {"nested": {"data": [1, 2, 3, {"deep": "value"}]}}},
        mock_data=True,
        metadata={"tags": ["edge", "case", "test"], "priority": 10}
    )
    
    # Test all job methods
    for job in [minimal_job, max_job]:
        # Test all status transitions
        original_status = job.status
        job.update_status(JobStatus.approved)
        job.update_status(JobStatus.running)
        job.update_status(JobStatus.completed)
        
        # Test properties
        assert job.is_terminal
        assert job.is_completed
        assert not job.is_running
        
        # Test string representations
        str(job)
        repr(job)
        
        # Test path operations
        if job.code_folder:
            job.resolved_code_folder
        if job.output_folder:
            job.resolved_output_folder
        
        # Test expiration
        job.is_expired
        
        # Test file operations
        job.code_files
        
        # Test save/load cycle
        job.save()
        job.load()
    
    # Test queue statistics
    stats = edge_queue.get_stats()
    assert stats["total_jobs"] >= 2
    
    # Test queue string representations
    str(edge_queue)
    repr(edge_queue)
